# Generated by Django 3.2.17 on 2023-02-02 21:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DemoModel1',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(max_length=255)),
                ('integer', models.IntegerField()),
                ('logic', models.BooleanField(default=False)),
                ('null_logic', models.NullBooleanField(default=None)),
                ('date', models.DateField()),
                ('timestamp', models.DateTimeField()),
                ('nullable', models.CharField(default=None, max_length=255, null=True)),
                ('choice', models.IntegerField(choices=[(1, 'Choice 1'), (2, 'Choice 2'), (3, 'Choice 3')])),
                ('j', models.JSONField(default={})),
                ('fk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='DemoModel2',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(blank=True, max_length=255, null=True)),
                ('integer', models.IntegerField(blank=True, null=True)),
                ('fk', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel1')),
            ],
        ),
        migrations.CreateModel(
            name='DemoModel3',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(blank=True, max_length=255, null=True)),
                ('integer', models.IntegerField(blank=True, null=True)),
                ('fk', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel1')),
            ],
        ),
        migrations.CreateModel(
            name='DemoModel5',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(blank=True, max_length=255, null=True)),
                ('integer', models.IntegerField(blank=True, null=True)),
                ('fk2', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel2')),
                ('fk3', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel3')),
            ],
        ),
        migrations.CreateModel(
            name='VeryLongNameModelDemoModel4',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('char', models.CharField(blank=True, max_length=255, null=True)),
                ('integer', models.IntegerField(blank=True, null=True)),
                ('fk2', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel2')),
                ('fk3', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='app1.demomodel3')),
            ],
        ),
    ]
